/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package org.processmining.xeslite.common;

import org.deckfour.xes.id.XID;

import com.google.common.primitives.Longs;

public final class XSeqID extends XID {

	private final long id;

	public XSeqID(long id) {
		this.id = id;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof XSeqID) {
			XSeqID other = (XSeqID) obj;
			return id == other.id;
		} else {
			return false;
		}
	}

	@Override
	public String toString() {
		return Long.toString(id);
	}

	@Override
	public Object clone() {
		return new XSeqID(id);
	}

	@Override
	public int hashCode() {
		return Longs.hashCode(id);
	}

	@Override
	public int compareTo(XID o) {
		if (o instanceof XSeqID) {
			return Longs.compare(this.id, ((XSeqID) o).id);
		} else {
			return -1; //TODO correct?
		}
	}

}